#version 410 core

in vec3 worldNormal;
in vec2 texCoord;
in vec3 worldPos;
in vec4 posLightSpace;

uniform sampler2D color_texture;
uniform sampler2DShadow shadowMap;

uniform vec3 cameraPosition;

struct DirectionalLight
{
   vec3 lightColor;
   vec3 lightPos;
   float ambientIntensity;
};

struct Material
{
   float specularIntensity;
   float specularPower;
   float transparency;
}; 

uniform DirectionalLight sunLight; 
uniform Material mat;

layout (location = 0) out vec4 fragColor;

float shadowValue(vec4 fragLightSpace) {

	 //return float(texture(shadowMap, fragLightSpace.xy).r < fragLightSpace.z);

	 float shadowMult = textureProj(shadowMap, fragLightSpace);

	 return shadowMult;
}

void main() {

	vec4 surfaceColor = texture(color_texture, texCoord);
	vec3 surfaceToLight = normalize(sunLight.lightPos - worldPos);
    vec3 surfaceToCamera = normalize(cameraPosition - worldPos);
	vec3 normal = normalize(worldNormal);

	//ambient
    vec3 ambient = sunLight.ambientIntensity * surfaceColor.rgb * sunLight.lightColor;

    //diffuse
    float diffuseCoefficient = max(0.0, dot(normal, normalize(sunLight.lightPos)));
    vec3 diffuse = diffuseCoefficient * surfaceColor.rgb * sunLight.lightColor;
    
    //specular
    float specularCoefficient = 0.0;
    if(diffuseCoefficient > 0.0)
        specularCoefficient = pow(max(0.0, dot(surfaceToCamera, reflect(surfaceToLight, normal))), mat.specularPower);
    vec3 specular = mat.specularIntensity * specularCoefficient * sunLight.lightColor;

	fragColor = vec4(ambient + (1.0 - shadowValue(posLightSpace)) * ( diffuse + specular), mat.transparency);
	//fragColor = vec4(vec3(texture(shadowMap, gl_FragCoord.xy / vec2(800,600)).r), 1.0) + tempvec * 0.000001;
	
}