#version 410 core

// used resources: 
// http://john-chapman-graphics.blogspot.co.at/2013/02/pseudo-lens-flare.html
// https://github.com/daanvanhasselt/lensflare

out vec4 color;
in vec2 uv;

uniform sampler2D image;
uniform sampler2D sun_texture;

uniform int ghost_amount = 6;
uniform float ghost_dispersal = 0.47f;
uniform float halo_width = 0.4f;
uniform vec2 dimensions = vec2(800, 600);
uniform vec3 chroma = vec3(0.8f, 0.5f, 0.6f);

vec3 textureDistorted(sampler2D tex, vec2 point, vec2 vector, vec3 distortion) {
	return vec3(
		texture2D(tex, point + vector).r * distortion.r,
		texture2D(tex, point + vector).g * distortion.g,
		texture2D(tex, point + vector).b * distortion.b
	);
}

void main()
{
	vec2 invUV = vec2(1.0) - uv;
	vec2 imageCenter = vec2(0.5);
	
	//vec2 ghostVec = (imageCenter - invUV) * ghost_dispersal;
	vec2 ghostVec = (imageCenter - uv) * ghost_dispersal;
	
	vec4 result = vec4(0.0);
	
	// create halos
	vec2 haloVec = normalize(imageCenter - uv) * halo_width;
	float weight = length(imageCenter - fract(uv + haloVec)) / length(imageCenter);
	weight = pow(1.0 - weight, 5.0);
	result += vec4(textureDistorted(sun_texture, uv, haloVec, chroma).rgb * weight, 1.0);
	
	haloVec = normalize(imageCenter - uv) * (halo_width/2);
	weight = length(imageCenter - fract(uv + haloVec)) / length(imageCenter);
	weight = pow(1.0 - weight, 5.0);
	result += vec4(textureDistorted(sun_texture, uv, haloVec, chroma).rgb * weight, 1.0);
	
	// create ghosts
	for(int i = 0; i < ghost_amount; i++){
	
		// with fract: additional ghosts appear if camera is centered on sun
		// without: ghosts become lines/squares if sun touches window edges
        vec2 offset = fract(uv + ghostVec * float(i));
		//vec2 offset = uv + ghostVec * float(i);
		
		// for some reason, lensflare goes haywire if chromadistortion is added to green here
        vec4 nochroma = vec4(texture(sun_texture, offset).rgb, 1.0);
		vec4 chroma = vec4(nochroma.r * chroma.r * i/ghost_amount, nochroma.g, nochroma.b * chroma.b * i/ghost_amount, 1.0);
		
		result += chroma;
	}
	
	// create ghosts - old method
	//for (int i = 0; i < ghost_amount; ++i) { 
	//	vec2 offset = fract(uv + ghostVec * float(i));
	//	result += texture(sun_texture, offset);
	//}
	
	// this fixes the scene getting super bright
	float brightness = dot(result.rgb, vec3(0.2126, 0.7152, 0.0722));
	if(brightness > 0.7)
		color = result;
	else
		color = vec4(0.0);
}