#version 410 core
// Effects: Normal Mapping, Shadow Calculation, Phong-Illumination, Textures

in vec3 position;
in vec3 normal;
in vec2 uv;

// Normal Mapping stuff
in vec3 tangent;
in vec3 bitangent;

out vec3 worldNormal;
out vec2 texCoord;
out vec3 worldPos;

//Vertex position in "LightSpace" for Shadows
out vec4 posLightSpace;
// Normal Mapping: Tangent * Bitangent * Normals Matrix
out mat3 TBN;


uniform mat4 model;
uniform mat4 view_project;

//LightSpace View * Projection Matrix for Shadows
uniform mat4 light_view_project;


void main() {
	
	gl_Position = view_project * model * vec4(position, 1.0);
	worldNormal = (model * vec4(normal, 0.0)).xyz;
	texCoord = uv;
	worldPos = vec3(model * vec4(position, 1));

	// Position of this vertex in Light Space
	posLightSpace = light_view_project * vec4(worldPos, 1.0);

	// Transform position to UV Space (because we are testing against points in textures)
	posLightSpace = vec4(posLightSpace.xyz*0.5+0.5, 1.0);

	// Construct TBN Matrix for normal Mapping
	vec3 T = normalize(vec3(model * vec4(tangent,   0.0)));
    vec3 B = normalize(vec3(model * vec4(bitangent, 0.0)));
    vec3 N = normalize(vec3(model * vec4(normal,    0.0)));
    TBN = mat3(T, B, N);

}